#include "interface.h"

#define ADD(s) textviewer->browser->add(s);
void show_readme_fr(TextViewer *textviewer)
{
textviewer->browser->clear();
	ADD("VBA Express.");
	ADD("");
	ADD("Auteur:   Achraf cherti, 2005-2006");
	ADD("Site Web: http://vbaexpress.tuxfamily.org/");
	ADD("Email:    achrafcherti@gmail.com");
	ADD("");
	ADD("@C4@bIntroduction");
	ADD("VBA Express est un frontend pour l'émulateur Gameboy Advance: VisualBoyAdvance");
	ADD("Il vous permettra de configurer aisément l'affichage, le son, les contrôles, etc. ");
	ADD("Sans oublier que vous pouvez lancer vos jeux par son biais sans passer par la");
	ADD("ligne de commande.");
	ADD("");
	ADD("Cette version a été testée avec VisualBoyAdvance version 1.7.1 et 1.7.2.");
	ADD("");
	ADD("@C4@bComment installer l'émulateur VisualBoyAdvance?");
	ADD("VBA Express a besoin de l'émulateur VisualBoyAdvance afin que vous puissiez émuler vos ");
	ADD("jeux Game Boy Advance. Il vous faudra le télécharger et installer depuis son site ");
	ADD("officiel: http://vba.ngemu.com");
	ADD("");
	ADD("Pour les gens pressés, voilà un lien direct pour télécharger la version 1.7.1");
	ADD("(version binaire):");
	ADD("http://prdownloads.sourceforge.net/vba/VisualBoyAdvance-1.7.1-SDL-linux-glibc22.tar.gz?download");
	ADD("");
	ADD("Dans l'archive VisualBoyAdvance-1.7.1-SDL-linux-glibc22.tar.gz l'exécutable \"VisualBoyAdvance\".");
	ADD("Il vous suffira de la copier dans /usr/bin (ou n'importe quel autre répertoire ");
	ADD("dans $PATH) afin que VBA Express puisse le détecter.");
	ADD("");
	ADD("@C4@bComment installer VBA Express");
	ADD("Pour installer le frontend il vous faudra copier le fichier exécutable");
	ADD("\"vbaexpress\" dans un des chemins définis dans la variable $PATH");
	ADD("par exemple \"/usr/bin\".");
	ADD("");
	ADD("@C4@bComment compiler le programme depuis la source...");
	ADD("Pour le compiler c'est très simple. Il vous faudra premièrement");
	ADD("quelques dépendances:");
	ADD("   - SDL     (www.libsdl.org)");
	ADD("   - Fltk    (www.fltk.org)");
	ADD("   - FLU     (www.osc.edu/~jbryan/FLU/)");
	ADD("");
	ADD("Une fois que vous aurez compilé et installé ces trois bibliothèques");
	ADD("vous pourrez compiler vous même Vba Express.");
	ADD("");
	ADD("Suposons que le fichier que vous avez téléchargé d'internet est");
	ADD("situé dans /home/achraf/vbaexpress-1.1-source.tar.gz");
	ADD("");
	ADD("Vous tapez (attention le $ ne compte pas...):");
	ADD("");
	ADD("$ cd /home/achraf");
	ADD("$ tar xvzf vbaexpress-1.1-source.tar.gz");
	ADD("");
	ADD("là vous aurez un nouveau répertoire nommé vbaexpress-1.1... vous y ");
	ADD("entrez et puis vous compilez le programme:");
	ADD("");
	ADD("$ cd vbaexpress-1.1");
	ADD("$ make");
	ADD("");
	ADD("une fois compilé, vous devrez vous loguer en root (vous entrez le mot");
	ADD("de passe après avoir tapé la commande \"su\"):");
	ADD("");
	ADD("$ su");
	ADD("password:");
	ADD("");
	ADD("et là vous pourrez installer VBA Express:");
	ADD("");
	ADD("$ make install");
	ADD("");
	ADD("");
	ADD("@C4@bQuels sont les outils utilisés pour le développement?");
	ADD("VBA Express est totalement programmé en C++. Compilé avec gcc et ");
	ADD("make. Le code source a été Edité avec gvim (http://www.gvim.org).");
	ADD("Le GUI est fait avec la libriarie Fltk (http://www.fltk.org).");
	ADD("File Browser avec la librairie FLU (complémentaire à Fltk)");
	ADD("L'image principale a été retouchée avec Gimp. Le système utilisé");
	ADD("pour le développement est Linux sous la distribution Fedora Core 4.");
	ADD("	");
	ADD("");
	textviewer->show();
}

#define ADD(s) textviewer->browser->add(s);
void show_readme_en(TextViewer *textviewer)
{
textviewer->browser->clear();
	ADD("VBA Express.");
	ADD("");
	ADD("Author:   Achraf cherti, 2005-2006");
	ADD("Website:  http://vbaexpress.tuxfamily.org/");
	ADD("Email:    achrafcherti@gmail.com");
	ADD("");
	ADD("@C4@bIntroduction");
	ADD("VBA Express is a VisualBoyAdvance frontend. It can help you:");
	ADD("* to configure easily the graphics, sounds, controls, paths...");
	ADD("* to run your games without console :-) but with the graphical \"Play!\" Button");
	ADD("");
	ADD("This version has been tested with VisualBoyAdvance version 1.7.1 and 1.7.2.");
	ADD("");
	ADD("@C4@bHow to install the emulator, VisualBoyAdvance?");
	ADD("Install the emulator:");
	ADD("---------------------");
	ADD("VBA Express requires the emulator (VisualBoyAdvance). It is used to run your");
	ADD("GameBoy Advance games. You can download it at: http://vba.ngemu.com");
	ADD("");
	ADD("For hasty people, you can download the binary archive for Linux directly at:");
	ADD("http://prdownloads.sourceforge.net/vba/VisualBoyAdvance-1.7.1-SDL-linux-glibc22.tar.gz?download");
	ADD("(it's version 1.7.1)");
	ADD("");
	ADD("In the archive VisualBoyAdvance-1.7.1-SDL-linux-glibc22.tar.gz you have some ");
	ADD("files. You must copy \"VisualBoyAdvance\" executable in /usr/bin (or in another ");
	ADD("path defined in the $PATH variable).");
	ADD("");
	ADD("IMPORTANT: VBA Express searches the VisualBoyAdvance in $PATH");
	ADD("");
	ADD("@C4@bHow to install VBA Express (the frontend):");
	ADD("You must copy \"vbaexpress\" executable in /usr/bin (or in another");
	ADD("path defined in the $PATH variable)");
	ADD("");
	ADD("@C4@bWhat's the tools used to create VBA Express?");
	ADD("VBA Express is programmed with C++ language. Compiled with gcc");
	ADD("and make. The source code has been edited with gvim (www.gvim.org).");
	ADD("The GUI with fltk (www.fltk.org) and FLU.");
	ADD("The OS used is Linux with the distribution: Fedora Core 4");
	ADD("");
	ADD("@C4@bHow to compile the source?");
	ADD("To compile the program from sources, you need some requirements:");
	ADD("   - SDL     (www.libsdl.org)");
	ADD("   - Fltk    (www.fltk.org)");
	ADD("   - FLU     (www.osc.edu/~jbryan/FLU/)");
	ADD("");
	ADD("In my example. You have the source archive in:");
	ADD("/home/achraf/vbaexpress-1.1-source.tar.gz");
	ADD("");
	ADD("Type (the $ is the shell... do not type it):");
	ADD("");
	ADD("$ cd /home/achraf");
	ADD("$ tar xvzf vbaexpress-1.1-source.tar.gz");
	ADD("");
	ADD("You have how \"vbaexpress-1.1\" directory. Now, type:");
	ADD("");
	ADD("$ cd vbaexpress-1.1");
	ADD("$ make");
	ADD("");
	ADD("And to install it type \"su\" to have root priviledges:");
	ADD("$ su");
	ADD("password:");
	ADD("");
	ADD("And now, you can install VBA Express with:");
	ADD("");
	ADD("$ make install");
	ADD("");
	textviewer->show();
}

